import bs
import bsUtils

gAchievements = []
gAchievementsToDisplay = []
gAchievementDisplayTimer = None
gCompletionBannerSlots = set()

def setCompletedAchievements(achs):
    #print 'ACHS WAS',bs.getConfig()['Achievements']
    # clear all then fill these in
    bs.getConfig()['Achievements'] = {}
    for aName in achs:
        getAchievement(aName).setComplete(announce=False,writeConfig=False)
    # write config in one fell swoop..
    bs.writeConfig()
    #print 'ACHS IS',bs.getConfig()['Achievements']

def getAchievement(name):
    try: return [a for a in gAchievements if a._name == name][0]
    except Exception: raise Exception("Invalid achievement name: '"+name+"'")

def getAchievementsForCoopLevel(level):
    return [a for a in gAchievements if a._level == level]

def _displayNextAchievement():
    global gAchievementDisplayTimer

    # pull the first achievement off the list and display it, or kill the display-timer if the list is empty
    if len(gAchievementsToDisplay) > 0:
        a,sound = gAchievementsToDisplay.pop(0)
        a.showCompletionBanner(sound)
        
    else:
        gAchievementDisplayTimer = None

class Achievement(object):
    def __init__(self,name,iconName,iconColor,level):
        self._name = name
        self._iconName = iconName
        self._iconColor = list(iconColor)+[1]
        self._level = level

    def getIconTexture(self,complete):
        return bs.getTexture(self._iconName if complete else 'achievementEmpty')

    def getIconColor(self,complete):
        return self._iconColor if complete else (1,1,1,0.6)

    def _getConfig(self):
        if not 'Achievements' in bs.getConfig(): bs.getConfig()['Achievements'] = {}
        if not self._name in bs.getConfig()['Achievements']:
            bs.getConfig()['Achievements'][self._name] = {'Complete':False}
        return bs.getConfig()['Achievements'][self._name]

    def isComplete(self):
        config = self._getConfig()
        return config['Complete']

    def setComplete(self,complete=True,announce=True,sound=True,writeConfig=True):
        import bsInternal
        config = self._getConfig()
        if complete != config['Complete']:
            config['Complete'] = complete
            if complete and announce:
                # only report it to the online service when initially achieving
                bsInternal._reportAchievement(self._name)
                # if we're being freshly complete, display/report it and whatnot
                gAchievementsToDisplay.append([self,sound])
                # if there's no achievement display timer going, kick one off
                # (if one's already running it will pick this up before it dies)
                global gAchievementDisplayTimer
                if gAchievementDisplayTimer is None:
                    gAchievementDisplayTimer = bs.Timer(1000,_displayNextAchievement,repeat=True,timeType='real')
                    _displayNextAchievement() # show the first immediately
            if writeConfig:
                bs.writeConfig()

    def getNameTranslated(self):
        val = bs.getResource('achievements')[self._name]['name']
        # name might have ${LEVEL} in it; in that case we substitute
        # the translated level name
        try:
            import bsCoopGame
            campaignName,campaignLevel = self._level.split(':')
            c = bsCoopGame.getCampaign(campaignName)
            n = c.getLevelDisplayName(campaignLevel)
            val = val.replace('${LEVEL}',n)
        except Exception,e:
            print 'EXC',e
        return val

    def getDescriptionTranslated(self):
        return bs.getResource('achievements')[self._name]['description']

    def getDescriptionCompleteTranslated(self):
        return bs.getResource('achievements')[self._name]['descriptionComplete']

    def createDisplay(self,activity,x,y,delay,inGame=False,outDelay=None,color=None):

        if inGame:
            objs = []
            outDelayFin = (delay+outDelay) if outDelay is not None else None
            if color is not None:
                c1 = (2.0*color[0],2.0*color[1],2.0*color[2],color[3])
                c2 = color
            else:
                c1 = (1.5,1.5,2,1.0)
                c2 = (0.8,0.8,1.0,1.0)

            objs.append(bsUtils.Image(activity,bs.Actor.RETAINED,
                                      self.getIconTexture(False),
                                      color=c1,
                                      position=(x-25,y+5),
                                      attach="topLeft",
                                      transition='fadeIn',
                                      transitionDelay=delay,
                                      transitionOutDelay=outDelayFin,
                                      scale=(40,40)))
            objs.append(bsUtils.Text(activity,bs.Actor.RETAINED,
                                     self.getNameTranslated(),
                                     maxWidth=300,
                                     position=(x,y+2),
                                     transition='fadeIn',
                                     scale=0.85,
                                     flatness=0.6,
                                     hAttach="left",
                                     vAttach="top",
                                     color=c2,
                                     transitionDelay=delay+50,
                                     transitionOutDelay=outDelayFin))
            objs.append(bsUtils.Text(activity,bs.Actor.RETAINED,
                                     self.getDescriptionTranslated(),
                                     maxWidth=400,
                                     position=(x,y-14),
                                     transition='fadeIn',
                                     hAttach="left",
                                     vAttach="top",
                                     scale=0.62,
                                     flatness=1.0,
                                     color=c2,
                                     transitionDelay=delay+100,
                                     transitionOutDelay=outDelayFin))
            # if we're transitioning out, kill ourselves after we're invised
            # if outDelayFin is not None:
            #     for obj in objs: bs.gameTimer(outDelayFin+6000,bs.WeakCall(obj.handleMessage,bs.DieMessage()))
                
        else:
            complete = self.isComplete()
            objs = []
            objs.append(bsUtils.Image(activity,bs.Actor.RETAINED,
                                      self.getIconTexture(complete),
                                      color=self.getIconColor(complete),
                                      position=(x-25,y+5),
                                      attach="center",
                                      transition='inRight',
                                      transitionDelay=delay,
                                      transitionOutDelay=None,
                                      scale=(40,40)))
            if complete:
                objs.append(bsUtils.Image(activity,bs.Actor.RETAINED,
                                          bs.getTexture('achievementOutline'),
                                          modelTransparent=bs.getModel('achievementOutline'),
                                          color=(2,1.4,0.4,1),
                                          position=(x-25,y+5),
                                          attach="center",
                                          transition='inRight',
                                          transitionDelay=delay,
                                          transitionOutDelay=None,
                                          scale=(40,40)))

            objs.append(bsUtils.Text(activity,bs.Actor.RETAINED,
                                     self.getNameTranslated(),
                                     maxWidth=300,
                                     position=(x,y+2),
                                     transition='inRight',
                                     scale=0.85,
                                     flatness=0.6,
                                     hAttach="center",
                                     vAttach="center",
                                     color=((0.8,0.93,0.8,1.0) if complete else (0.6,0.6,0.6,0.4)),
                                     transitionDelay=delay+50,
                                     transitionOutDelay=None))
            objs.append(bsUtils.Text(activity,bs.Actor.RETAINED,
                                     self.getDescriptionCompleteTranslated() if complete else self.getDescriptionTranslated(),
                                     maxWidth=400,
                                     position=(x,y-14),
                                     transition='inRight',
                                     hAttach="center",
                                     vAttach="center",
                                     scale=0.62,
                                     flatness=1.0,
                                     color=((0.6,0.6,0.6,1.0) if complete else (0.6,0.6,0.6,0.4)),
                                     transitionDelay=delay+100,
                                     transitionOutDelay=None))
        return objs

    def _removeBannerSlot(self):
        gCompletionBannerSlots.remove(self._completionBannerSlot)

    def showCompletionBanner(self,sound=True):

        # just piggy-pack onto any current activity... (should we pass one in instead?..)
        try: activity = bs.getSession().getActivity()
        except Exception: activity = None

        if activity is None:
            print 'showCompletionBanner() called with no current activity!'
            return

        if sound: bs.playSound(bs.getSound('achievement'))
        else: bs.gameTimer(500,bs.Call(bs.playSound,bs.getSound('ding')))
        
        # remove us from that slot when we close
        yOffs = 0
        inTime = 300
        outTime = 3500

        # find the first free slot
        for i in range(10):
            if not i in gCompletionBannerSlots:
                gCompletionBannerSlots.add(i)
                self._completionBannerSlot = i
                bs.realTimer(inTime+outTime,self._removeBannerSlot)
                break

        yOffs = 110*self._completionBannerSlot

        objs = []
        objs.append(bsUtils.Image(activity,bs.Actor.RETAINED,
                                  bs.getTexture('shadow'),
                                  position=(-30,30+yOffs),
                                  attach='bottomCenter',
                                  transition='inBottom',
                                  transitionDelay=inTime,
                                  transitionOutDelay=outTime,
                                  color=(0.0,0.1,0,1),
                                  scale=(500,150)))
        obj = bsUtils.Image(activity,bs.Actor.RETAINED,
                            bs.getTexture('light'),
                            position=(-180,60+yOffs),
                            attach='bottomCenter',
                            transition='inBottom',
                            transitionDelay=inTime,
                            transitionOutDelay=outTime,
                            color=(1.8,1.8,1.0,0.0),
                            scale=(40,300))
        objs.append(obj)

        obj.node.premultiplied = True
        c = bs.newNode("combine",owner=obj.node)
        bsUtils.animate(c,'input0',{inTime:0, inTime+400:30, inTime+500:40, inTime+600:30, inTime+2000:0})
        bsUtils.animate(c,'input1',{inTime:0, inTime+400:200, inTime+500:500, inTime+600:200, inTime+2000:0})
        c.connectAttr('output',obj.node,'scale')
        bsUtils.animate(obj.node,'rotate',{0:0.0,350:360.0},loop=True)
        obj = bsUtils.Image(activity,bs.Actor.RETAINED,
                            self.getIconTexture(True),
                            position=(-180,60+yOffs),
                            attach='bottomCenter',
                            transition='inBottom',
                            transitionDelay=inTime,
                            transitionOutDelay=outTime,
                            scale=(100,100))
        objs.append(obj)

        # flash
        color = self.getIconColor(True)
        c = bs.newNode("combine",owner=obj.node,attrs={'size':3})
        keys = {inTime:1.0*color[0], inTime+400:1.5*color[0], inTime+500:6.0*color[0], inTime+600:1.5*color[0], inTime+2000:1.0*color[0]}
        bsUtils.animate(c,'input0',keys)
        keys = {inTime:1.0*color[1], inTime+400:1.5*color[1], inTime+500:6.0*color[1], inTime+600:1.5*color[1], inTime+2000:1.0*color[1]}
        bsUtils.animate(c,'input1',keys)
        keys = {inTime:1.0*color[2], inTime+400:1.5*color[2], inTime+500:6.0*color[2], inTime+600:1.5*color[2], inTime+2000:1.0*color[2]}
        bsUtils.animate(c,'input2',keys)
        c.connectAttr('output',obj.node,'color')

        obj = bsUtils.Image(activity,bs.Actor.RETAINED,
                            bs.getTexture('achievementOutline'),
                            modelTransparent=bs.getModel('achievementOutline'),
                            position=(-180,60+yOffs),
                            attach='bottomCenter',
                            transition='inBottom',
                            transitionDelay=inTime,
                            transitionOutDelay=outTime,
                            scale=(100,100))
        # flash
        color = (2,1.4,0.4,1)
        c = bs.newNode("combine",owner=obj.node,attrs={'size':3})
        keys = {inTime:1.0*color[0], inTime+400:1.5*color[0], inTime+500:6.0*color[0], inTime+600:1.5*color[0], inTime+2000:1.0*color[0]}
        bsUtils.animate(c,'input0',keys)
        keys = {inTime:1.0*color[1], inTime+400:1.5*color[1], inTime+500:6.0*color[1], inTime+600:1.5*color[1], inTime+2000:1.0*color[1]}
        bsUtils.animate(c,'input1',keys)
        keys = {inTime:1.0*color[2], inTime+400:1.5*color[2], inTime+500:6.0*color[2], inTime+600:1.5*color[2], inTime+2000:1.0*color[2]}
        bsUtils.animate(c,'input2',keys)
        c.connectAttr('output',obj.node,'color')
        objs.append(obj)

        objs.append(bsUtils.Text(activity,bs.Actor.RETAINED,
                                 bs.getResource('achievementText')+':',
                                 position=(-120,91+yOffs),
                                 vAttach='bottom',
                                 transition='inBottom',
                                 flatness=0.5,
                                 transitionDelay=inTime,
                                 transitionOutDelay=outTime,
                                 color=(1,1,1,0.8),
                                 scale=0.65))

        obj = bsUtils.Text(activity,bs.Actor.RETAINED,
                           self.getNameTranslated(),
                           position=(-120,50+yOffs),
                           vAttach='bottom',
                           transition='inBottom',
                           flatness=0.5,
                           transitionDelay=inTime,
                           transitionOutDelay=outTime,
                           flash=True,
                           color=(1,0.8,0,1.0),
                           scale=1.5)
        objs.append(obj)

        objs.append(bsUtils.Text(activity,bs.Actor.RETAINED,
                                 self.getDescriptionCompleteTranslated(),
                                 position=(-120,30+yOffs),
                                 vAttach='bottom',
                                 transition='inBottom',
                                 flatness=0.5,
                                 transitionDelay=inTime,
                                 transitionOutDelay=outTime,
                                 color=(1.0,0.7,0.5,1.0),
                                 scale=0.8))

        for obj in objs: bs.gameTimer(outTime+1000,bs.WeakCall(obj.handleMessage,bs.DieMessage()))

#5
gAchievements.append(Achievement('Onslaught Master','achievementMedalSmall',(0.7,1,0.7),'Challenges:Onslaught'))
#15
gAchievements.append(Achievement('Onslaught Wizard','achievementMedalMedium',(0.7,1.0,0.7),'Challenges:Onslaught'))
#30
gAchievements.append(Achievement('Onslaught God','achievementMedalLarge',(0.7,1.0,0.7),'Challenges:Onslaught'))

#5
gAchievements.append(Achievement('Runaround Master','achievementMedalSmall',(1.0,1.0,1.2),'Challenges:Runaround'))
#15
gAchievements.append(Achievement('Runaround Wizard','achievementMedalMedium',(1.0,1.0,1.2),'Challenges:Runaround'))
#30
gAchievements.append(Achievement('Runaround God','achievementMedalLarge',(1.0,1.0,1.2),'Challenges:Runaround'))
        
#5
gAchievements.append(Achievement('Onslaught Training Victory','achievementOnslaught',(1,1,1),'Default:Onslaught Training'))
#5
gAchievements.append(Achievement('Off You Go Then','achievementOffYouGo',(1,1.1,1.3),'Default:Onslaught Training'))
#10
gAchievements.append(Achievement('Boxer','achievementBoxer',(1,0.6,0.6),'Default:Onslaught Training'))

#10
gAchievements.append(Achievement('Rookie Onslaught Victory','achievementOnslaught',(0.5,1.4,0.6),'Default:Rookie Onslaught'))
#10
gAchievements.append(Achievement('Mine Games','achievementMine',(1,1,1.4),'Default:Rookie Onslaught'))
#15
gAchievements.append(Achievement('Flawless Victory','achievementFlawlessVictory',(1,1,1),'Default:Rookie Onslaught'))

#10
gAchievements.append(Achievement('Rookie Football Victory','achievementFootballVictory',(1.0,1,0.6),'Default:Rookie Football'))
#10
gAchievements.append(Achievement('Super Punch','achievementSuperPunch',(1,1,1.8),'Default:Rookie Football'))
#15
gAchievements.append(Achievement('Rookie Football Shutout','achievementFootballShutout',(1,1,1),'Default:Rookie Football'))

#15
gAchievements.append(Achievement('Pro Onslaught Victory','achievementOnslaught',(0.3,1,2.0),'Default:Pro Onslaught'))
#15
gAchievements.append(Achievement('Boom Goes the Dynamite','achievementTNT',(1.4,1.2,0.8),'Default:Pro Onslaught'))
#20
gAchievements.append(Achievement('Pro Boxer','achievementBoxer',(2,2,0),'Default:Pro Onslaught'))

#15
gAchievements.append(Achievement('Pro Football Victory','achievementFootballVictory',(1.3,1.3,2.0),'Default:Pro Football'))
#15
gAchievements.append(Achievement('Super Mega Punch','achievementSuperPunch',(2,1,0.6),'Default:Pro Football'))
#20
gAchievements.append(Achievement('Pro Football Shutout','achievementFootballShutout',(0.7,0.7,2.0),'Default:Pro Football'))

#15
gAchievements.append(Achievement('Pro Runaround Victory','achievementRunaround',(1,1,1),'Default:Pro Runaround'))
#20
gAchievements.append(Achievement('Precision Bombing','achievementCrossHair',(1,1,1.3),'Default:Pro Runaround'))
#25
gAchievements.append(Achievement('The Wall','achievementWall',(1,0.7,0.7),'Default:Pro Runaround'))

#30
gAchievements.append(Achievement('Uber Onslaught Victory','achievementOnslaught',(2,2,1),'Default:Uber Onslaught'))
#30
gAchievements.append(Achievement('Gold Miner','achievementMine',(2,1.6,0.2),'Default:Uber Onslaught'))
#30
gAchievements.append(Achievement('TNT Terror','achievementTNT',(2,1.8,0.3),'Default:Uber Onslaught'))

#30
gAchievements.append(Achievement('Uber Football Victory','achievementFootballVictory',(1.8,1.4,0.3),'Default:Uber Football'))
#30
gAchievements.append(Achievement('Got the Moves','achievementGotTheMoves',(2,1,0),'Default:Uber Football'))
#40
gAchievements.append(Achievement('Uber Football Shutout','achievementFootballShutout',(2,2,0),'Default:Uber Football'))

#30
gAchievements.append(Achievement('Uber Runaround Victory','achievementRunaround',(1.5,1.2,0.2),'Default:Uber Runaround'))
#40
gAchievements.append(Achievement('The Great Wall','achievementWall',(2,1.7,0.4),'Default:Uber Runaround'))
#40
gAchievements.append(Achievement('Stayin\' Alive','achievementStayinAlive',(2,2,1),'Default:Uber Runaround'))

#20
gAchievements.append(Achievement('Last Stand Master','achievementMedalSmall',(2,1.5,0.3),'Default:The Last Stand'))
#40
gAchievements.append(Achievement('Last Stand Wizard','achievementMedalMedium',(2,1.5,0.3),'Default:The Last Stand'))
#60
gAchievements.append(Achievement('Last Stand God','achievementMedalLarge',(2,1.5,0.3),'Default:The Last Stand'))

# just a test...

# def foo():
#     gAchievements[0].setComplete(False)
#     gAchievements[1].setComplete(False)
#     gAchievements[2].setComplete(False)

# def foo2():
#     gAchievements[0].setComplete()
#     gAchievements[1].setComplete()
#     gAchievements[2].setComplete()

# bs.realTimer(4000+1000,foo)
# bs.realTimer(4000+3000,foo2)
# bs.realTimer(4000,foo)
# bs.realTimer(7000,foo2)

