import bs
import bsGame
import bsMap
import time
import os
import math

def _safeSetAttr(node,attr,value):
    if node.exists(): setattr(node,attr,value)
            
class ButtonPress(object):
    def __init__(self,button,delay=0,release=True,releaseDelay=0):
        self._button = button
        self._delay = delay
        self._release = release
        self._releaseDelay = releaseDelay
    def run(self,a):
        s = a._currentSpaz
        if self._button == 'punch':
            call = s.onPunchPress
            releaseCall = s.onPunchRelease
            img = a._punchImage
            color = a._punchImageColor
        elif self._button == 'jump':
            call = s.onJumpPress
            releaseCall = s.onJumpRelease
            img = a._jumpImage
            color = a._jumpImageColor
        elif self._button == 'bomb':
            call = s.onBombPress
            releaseCall = s.onBombRelease
            img = a._bombImage
            color = a._bombImageColor
        elif self._button == 'pickUp':
            call = s.onPickUpPress
            releaseCall = s.onPickUpRelease
            img = a._pickUpImage
            color = a._pickUpImageColor
        elif self._button == 'run':
            call = s.onRunPress
            releaseCall = s.onRunRelease
            img = None
            color = None
        else: raise Exception("invalid button: "+self._button)

        brightness = 4.0
        if color is not None:
            cBright = list(color)
            cBright[0] *= brightness
            cBright[1] *= brightness
            cBright[2] *= brightness
        
        if self._delay == 0:
            call()
            if img is not None:
                img.color = cBright
        else:
            bs.gameTimer(self._delay,call)
            if img is not None:
                bs.gameTimer(self._delay,bs.Call(_safeSetAttr,img,'color',cBright))
        if self._release:
            if self._delay == 0 and self._releaseDelay == 0:
                releaseCall()
            else:
                bs.gameTimer(self._delay+self._releaseDelay,releaseCall)
            if img is not None:
                bs.gameTimer(self._delay+self._releaseDelay+100,bs.Call(_safeSetAttr,img,'color',color))


class ButtonRelease(object):
    def __init__(self,button,delay=0):
        self._button = button
        self._delay = delay
    def run(self,a):
        s = a._currentSpaz
        if self._button == 'punch':
            call = s.onPunchRelease
            img = a._punchImage
            color = a._punchImageColor
        elif self._button == 'jump':
            call = s.onJumpRelease
            img = a._jumpImage
            color = a._jumpImageColor
        elif self._button == 'bomb':
            call = s.onBombRelease
            img = a._bombImage
            color = a._bombImageColor
        elif self._button == 'pickUp':
            call = s.onPickUpPress
            img = a._pickUpImage
            color = a._pickUpImageColor
        elif self._button == 'run':
            call = s.onRunRelease
            img = None
            color = None
        else: raise Exception("invalid button: "+self._button)
        if self._delay == 0: call()
        else: bs.gameTimer(self._delay,call)
        if img is not None:
            bs.gameTimer(self._delay+100,bs.Call(_safeSetAttr,img,'color',color))

class TutorialActivity(bs.Activity):

    def __init__(self,session,settings={}):
        bs.Activity.__init__(self,session,settings)

        self._allowPausing = True
        self._allowKickIdlePlayers = False

        self._mapType = bsMap.RampageMap
        self._mapType.preload()

        self._jumpButtonTex = bs.getTexture('buttonJump')
        self._pickUpButtonTex = bs.getTexture('buttonPickUp')
        self._bombButtonTex = bs.getTexture('buttonBomb')
        self._punchButtonTex = bs.getTexture('buttonPunch')
        self._R = bs.getResource('tutorial')
        self._haveSkipped = False
        self._stickImagePositionX = self._stickImagePositionY = 0.0

        self._spawnSound = bs.getSound('spawn')
        
    def onTransitionIn(self):
        bs.Activity.onTransitionIn(self)
        bs.playMusic('CharSelect')
        self._map = self._mapType(self,bs.Actor.UNRETAINED)

    def onBegin(self):
        bs.Activity.onBegin(self)
        #bs.gameTimer(5000,bs.WeakCall(self.end))

        # pos = (0,5.5,-4.4)
        # s = self._spaz1 = bs.Spaz(self,bs.Actor.UNRETAINED)
        # s.handleMessage(bs.StandMessage(pos,0))

        if 0:
            # buttons on top
            textY = 140
            buttonsY = 250
        else:
            # buttons on bottom
            textY = 260
            buttonsY = 160
        
        # need different versions of this: taps/buttons/keys
        t = self._text = bs.newNode('text',
                                        owner=self,
                                        attrs={'text':'',
                                               'scale':[1.9],
                                               'translate':(0,textY),
                                               'maxWidth':500,
                                               'flatness':0.0,
                                               'shadow':0.5,
                                               'hAlign':'center',
                                               'vAlign':'center',
                                               'vAttach':'center'})
        #bs.animate(t,'opacity',{1000:0.0,1200:1.0})
        

        # need different versions of this: taps/buttons/keys
        t = self._skipText = bs.newNode('text',
                                        owner=self,
                                        attrs={'text':self._R.toSkipPressAnythingText,
                                               'scale':[1.1],
                                               'translate':(0,30),
                                               'hAlign':'center',
                                               'vAlign':'center',
                                               'vAttach':'bottom'})
        bs.animate(t,'opacity',{1000:0.0,2000:0.7})
        t = self._skipCountText = bs.newNode('text',
                                                 owner=self,
                                                 attrs={'text':'',
                                                        'scale':[1.4],
                                                        'translate':(0,70),
                                                        'hAlign':'center',
                                                        'vAlign':'center',
                                                        'vAttach':'bottom'})

        env = bs.getEnvironment()
        ouya = (env['platform'] == 'android' and env['subplatform'] == 'ouya')
        
        self._scale = scale = 0.6
        centerOffs = 130.0*scale
        offs = 65.0*scale
        position=(0,buttonsY)
        imageSize = 90.0*scale
        imageSize2 = 220.0*scale
        nubSize = 110.0*scale
        p = (position[0]+centerOffs,position[1]-offs)

        def _sc(r,g,b,a=None):
            return (0.6*r,0.6*g,0.6*b)
        
        self._jumpImageColor = c = _sc(0.4,1,0.4)
        self._jumpImage = bs.newNode('image',owner=self,
                               attrs={'texture':self._jumpButtonTex,'absoluteScale':True,
                                       'translate':p,'scale':(imageSize,imageSize),'color':c})
        p = (position[0]+centerOffs-offs,position[1])
        self._punchImageColor = c = _sc(0.2,0.6,1) if ouya else _sc(1,0.7,0.3)
        self._punchImage = bs.newNode('image',owner=self,
                               attrs={'texture':bs.getTexture('buttonPunch'),'absoluteScale':True,
                                      'translate':p,'scale':(imageSize,imageSize),'color':c})
        p = (position[0]+centerOffs+offs,position[1])
        self._bombImageColor = c = _sc(1,0.3,0.3)
        self._bombImage = bs.newNode('image',owner=self,
                               attrs={'texture':bs.getTexture('buttonBomb'),'absoluteScale':True,
                                      'translate':p,'scale':(imageSize,imageSize),'color':c})
        p = (position[0]+centerOffs,position[1]+offs)
        self._pickUpImageColor = c = _sc(1,0.8,0.3) if ouya else _sc(0.8,0.5,1)
        self._pickUpImage = bs.newNode('image',owner=self,
                                 attrs={'texture':bs.getTexture('buttonPickUp'),'absoluteScale':True,
                                        'translate':p,'scale':(imageSize,imageSize),'color':c})

        self._stickBasePosition = p = (position[0]-centerOffs,position[1])
        self._stickBaseImageColor = c = (0.25,0.25,0.25,1.0)
        self._stickBaseImage = bs.newNode('image',owner=self,
                                          attrs={'texture':bs.getTexture('nub'),'absoluteScale':True,
                                                 'translate':p,'scale':(imageSize2,imageSize2),'color':c})
        self._stickNubPosition = p = (position[0]-centerOffs,position[1])
        self._stickNubImageColor = c = (0.4,0.4,0.4,1.0)
        self._stickNubImage = bs.newNode('image',owner=self,
                                          attrs={'texture':bs.getTexture('nub'),'absoluteScale':True,
                                                 'translate':p,'scale':(nubSize,nubSize),'color':c})
        self._controlUINodes = [self._jumpImage,self._punchImage,self._bombImage,self._pickUpImage,
                              self._stickBaseImage,self._stickNubImage]
        for n in self._controlUINodes:
            n.opacity = 0.0

        self._spazzes = {}
        
        # class Delay(object):
        #     def __init__(self,time):
        #         self._time = time
        #     def run(self,activity):
        #         return self._timee

        # class SetText(object):
        #     def __init__(self,text):
        #         self._text = text
        #     def run(self,activity):
        #         activity._text.text = self._text

        # self._entries = [Delay(1000),
        #                  SetText('Hi there!'),
        #                  Delay(2000),
        #                  SetText('Stuff and things'),
        #                  Delay(2000),
        #                  SetText('More stuff and whatnot')
        #              ]
        # self._testFile = '/Users/ericf/Library/Containers/net.froemling.bombsquad/Data/Library/Application Support/BombSquad/foo.py'
        # self._testFileModTime = os.path.getmtime(self._testFile)

        self._readEntries()
        # self._runNextEntry()

    def _setStickImagePosition(self,x,y):

        # clamp this to a circle
        lenSquared = x*x+y*y;
        if lenSquared > 1.0:
            l = math.sqrt(lenSquared)
            mult = 1.0/l;
            x *= mult;
            y *= mult;
        
        self._stickImagePositionX = x
        self._stickImagePositionY = y
        offs = 50.0
        p = [self._stickNubPosition[0]+x*offs*self._scale,
             self._stickNubPosition[1]+y*offs*self._scale]
        c = list(self._stickNubImageColor)
        if abs(x) > 0.1 or abs(y) > 0.1:
            c[0] *= 2.0
            c[1] *= 4.0
            c[2] *= 2.0
        self._stickNubImage.translate = p
        self._stickNubImage.color = c
        c = list(self._stickBaseImageColor)
        if abs(x) > 0.1 or abs(y) > 0.1:
            c[0] *= 1.5
            c[1] *= 1.5
            c[2] *= 1.5
        self._stickBaseImage.color = c
        
    def _readEntries(self):
        self._entries = []
        try:
            #print 'starting..'
            # f = open('/Users/ericf/Library/Containers/net.froemling.bombsquad/Data/Library/Application Support/BombSquad/foo.py')
            # s = f.read()
            # f.close()
            # exec(compile(s, 'foo.py', 'exec'))
            #execfile(self._testFile)

            class Reset(object):
                def __init__(self):
                    pass
                def run(self,a):
                    a._text.text = ''
                    for spaz in a._spazzes.values():
                        spaz.handleMessage(bs.DieMessage(immediate=True))
                    a._spazzes = {}
                    a._currentSpaz = None
                    for n in a._controlUINodes:
                        n.opacity = 0.0
                    a._setStickImagePosition(0,0)

            class SetSpeed(object):
                def __init__(self,speed):
                    self._speed = speed
                def run(self,a):
                    import bsInternal
                    bsInternal._setGameSpeed(self._speed)

            class RemoveGloves(object):
                def __init__(self):
                    pass
                def run(self,a):
                    #print 'WOuld remove gloves'
                    a._currentSpaz._glovesWearOff()

            # grumble - sometimes our real punches aren't landing due to simulator
            # variance so we force the issue if need be..
            class FakePunch(object):
                def __init__(self):
                    pass
                def run(self,a):
                    print 'WOULD FAKE PUNCH'

            class KillSpaz(object):
                def __init__(self,num,explode=False):
                    self._num = num
                    self._explode = explode
                def run(self,a):
                    #a._spazzes[self._num].hitPoints *= 0.1
                    if self._explode:
                        a._spazzes[self._num].shatter()
                    del a._spazzes[self._num]

            class SpawnSpaz(object):
                def __init__(self,num,position,color=(1,1,1),makeCurrent=False,relativeTo=None,name='',flash=True):
                    self._num = num
                    self._position = position
                    self._makeCurrent = makeCurrent
                    self._color = color
                    self._relativeTo = relativeTo
                    self._name = name
                    self._flash = flash
                def run(self,a):

                    # if they gave a 'relative to' spaz, position is relative to them
                    if self._relativeTo is not None:
                        theirPos = a._spazzes[self._relativeTo].node.translate
                        pos = (theirPos[0]+self._position[0],
                               theirPos[1]+self._position[1],
                               theirPos[2]+self._position[2])
                    else: pos = self._position

                    # if there's already a spaz at this spot, insta-kill it
                    if self._num in a._spazzes:
                        a._spazzes[self._num].handleMessage(bs.DieMessage(immediate=True))

                    s = a._spazzes[self._num] = bs.Spaz(a,bs.Actor.UNRETAINED,color=self._color,startInvincible=self._flash)
                    s.node.name = self._name
                    s.node.nameColor = self._color
                    s.handleMessage(bs.StandMessage(pos,0))
                    if self._makeCurrent:
                        a._currentSpaz = s
                    bs.playSound(a._spawnSound,position=pos)

            class Powerup(object):
                def __init__(self,num,position,color=(1,1,1),makeCurrent=False,relativeTo=None):
                    self._position = position
                    self._relativeTo = relativeTo
                def run(self,a):
                    # if they gave a 'relative to' spaz, position is relative to them
                    if self._relativeTo is not None:
                        theirPos = a._spazzes[self._relativeTo].node.translate
                        pos = (theirPos[0]+self._position[0],
                               theirPos[1]+self._position[1],
                               theirPos[2]+self._position[2])
                    else: pos = self._position

                    bs.Powerup(a,bs.Actor.RETAINED,translate=pos,powerupType='punch')

            class Delay(object):
                def __init__(self,time):
                    self._time = time
                def run(self,a):
                    return self._time

            class End(object):
                def __init__(self):
                    pass
                def run(self,a):
                    a.end()
                
            class Move(object):
                def __init__(self,x,y):
                    self._x = float(x)
                    self._y = float(y)
                def run(self,a):
                    s = a._currentSpaz
                    # FIXME - game should take floats for this..
                    xClamped = max(-32767,min(32767,int(self._x*32768.0)))
                    yClamped = max(-32767,min(32767,int(self._y*32768.0)))
                    #print 'MOVING',xClamped,yClamped
                    s.onMoveLeftRight(xClamped)
                    s.onMoveUpDown(yClamped)
                    a._setStickImagePosition(self._x,self._y)

            class MoveLR(object):
                def __init__(self,x):
                    self._x = float(x)
                def run(self,a):
                    s = a._currentSpaz
                    # FIXME - game should take floats for this..
                    xClamped = max(-32767,min(32767,int(self._x*32768.0)))
                    s.onMoveLeftRight(xClamped)
                    a._setStickImagePosition(self._x,a._stickImagePositionY)

            class MoveUD(object):
                def __init__(self,y):
                    self._y = float(y)
                def run(self,a):
                    s = a._currentSpaz
                    # FIXME - game should take floats for this..
                    yClamped = max(-32767,min(32767,int(self._y*32768.0)))
                    #print 'MOVING UD',yClamped
                    #print 'MOVING',xClamped,yClamped
                    s.onMoveUpDown(yClamped)
                    a._setStickImagePosition(a._stickImagePositionX,self._y)

            class Bomb(ButtonPress):
                def __init__(self,delay=0,release=True,releaseDelay=500):
                    ButtonPress.__init__(self,'bomb',delay=delay,release=release,releaseDelay=releaseDelay)

            class Jump(ButtonPress):
                def __init__(self,delay=0,release=True,releaseDelay=500):
                    ButtonPress.__init__(self,'jump',delay=delay,release=release,releaseDelay=releaseDelay)

            class Punch(ButtonPress):
                def __init__(self,delay=0,release=True,releaseDelay=500):
                    ButtonPress.__init__(self,'punch',delay=delay,release=release,releaseDelay=releaseDelay)

            class PickUp(ButtonPress):
                def __init__(self,delay=0,release=True,releaseDelay=500):
                    ButtonPress.__init__(self,'pickUp',delay=delay,release=release,releaseDelay=releaseDelay)

            class Run(ButtonPress):
                def __init__(self,delay=0,release=True,releaseDelay=500):
                    ButtonPress.__init__(self,'run',delay=delay,release=release,releaseDelay=releaseDelay)

            class BombRelease(ButtonRelease):
                def __init__(self,delay=0):
                    ButtonRelease.__init__(self,'bomb',delay=delay)

            class JumpRelease(ButtonRelease):
                def __init__(self,delay=0):
                    ButtonRelease.__init__(self,'jump',delay=delay)

            class PunchRelease(ButtonRelease):
                def __init__(self,delay=0):
                    ButtonRelease.__init__(self,'punch',delay=delay)

            class PickUpRelease(ButtonRelease):
                def __init__(self,delay=0):
                    ButtonRelease.__init__(self,'pickUp',delay=delay)

            class RunRelease(ButtonRelease):
                def __init__(self,delay=0):
                    ButtonRelease.__init__(self,'run',delay=delay)


            class ShowControls(object):
                def __init__(self):
                    pass
                def run(self,a):
                    for n in a._controlUINodes:
                        bs.animate(n,'opacity',{0:0.0,1000:1.0})

            class Text(object):
                def __init__(self,text):
                    self._text = text
                def run(self,a):
                    pass
                    a._text.text = self._text

            class PrintPos(object):
                def __init__(self,spazNum=None):
                    self._spazNum = spazNum
                def run(self,a):
                    if self._spazNum is None:
                        s = a._currentSpaz
                    else:
                        s = a._spazzes[self._spazNum]
                    t = list(s.node.translate)
                    print 'RestorePos('+str((t[0],t[1]-1.0,t[2]))+'),'

            class RestorePos(object):
                def __init__(self,pos):
                    self._pos = pos
                def run(self,a):
                    s = a._currentSpaz
                    s.handleMessage(bs.StandMessage(self._pos,0))

            class Celebrate(object):
                def __init__(self,celebrateType='both',spazNum=None,duration=1000):
                    self._spazNum = spazNum
                    self._celebrateType = celebrateType
                    self._duration = duration
                def run(self,a):
                    if self._spazNum is None:
                        s = a._currentSpaz
                    else:
                        s = a._spazzes[self._spazNum]
                    if self._celebrateType == 'right':
                        s.node.handleMessage('celebrateR',self._duration)
                    elif self._celebrateType == 'left':
                        s.node.handleMessage('celebrateL',self._duration)
                    elif self._celebrateType == 'both':
                        s.node.handleMessage('celebrate',self._duration)
                    else: raise Exception("invalid celebrate type "+self._celebrateType)

            R = bs.getResource('tutorial')
            self._entries = [Reset(),
                             #SetSpeed(100.0),
                             SpawnSpaz(0,(0,5.5,-3.0),makeCurrent=True),
                             Delay(1000),
                             Text(R.phrase01Text), # hi there
                             Celebrate('left'),
                             Delay(2000),
                             Text(R.phrase02Text), # welcome to bombsquad

                             Delay(80),Run(release=False),Jump(release=False),MoveLR(1),MoveUD(0),Delay(70),RunRelease(),JumpRelease(),Delay(60),MoveUD(1),Delay(30),MoveLR(0),Delay(90),MoveLR(-1),Delay(20),MoveUD(0),
                             Delay(70),MoveUD(-1),Delay(20),MoveLR(0),Delay(80),MoveUD(0),
                             Delay(1500),

                             Text(R.phrase03Text), # here's a few tips
                             Delay(1000),
                             ShowControls(),
                             Delay(1000),
                             Jump(),
                             Delay(1000),
                             Jump(),
                             Delay(1000),
                             Text(R.phrase04Text), # learn to harness the power of physics

                             Delay(20),MoveUD(0),Delay(60),MoveLR(0),Delay(10),MoveLR(0),MoveUD(0),Delay(10),MoveLR(0),MoveUD(0),Delay(20),MoveUD(-0.0575579),Delay(10),MoveUD(-0.207831),Delay(30),MoveUD(-0.309793),Delay(10),MoveUD(-0.474502),
                             Delay(10),MoveLR(0.00390637),MoveUD(-0.647053),Delay(20),MoveLR(-0.0745262),MoveUD(-0.819605),Delay(10),MoveLR(-0.168645),MoveUD(-0.937254),Delay(30),MoveLR(-0.294137),MoveUD(-1),Delay(10),MoveLR(-0.411786),Delay(10),MoveLR(-0.639241),
                             Delay(30),MoveLR(-0.75689),Delay(10),MoveLR(-0.905911),Delay(20),MoveLR(-1),Delay(50),MoveUD(-0.960784),Delay(20),MoveUD(-0.819605),MoveUD(-0.61568),Delay(20),MoveUD(-0.427442),Delay(20),MoveUD(-0.231361),Delay(10),MoveUD(-0.00390637),Delay(30),MoveUD(0.333354),
                             MoveUD(0.584338),Delay(20),MoveUD(0.764733),Delay(30),MoveLR(-0.803949),MoveUD(0.913755),Delay(10),MoveLR(-0.647084),MoveUD(0.992187),Delay(20),MoveLR(-0.435316),MoveUD(1),Delay(20),MoveLR(-0.168645),MoveUD(0.976501),
                             MoveLR(0.0744957),MoveUD(0.905911),Delay(20),MoveLR(0.270577),MoveUD(0.843165),Delay(20),MoveLR(0.435286),MoveUD(0.780419),Delay(10),MoveLR(0.66274),MoveUD(0.647084),Delay(30),MoveLR(0.803919),MoveUD(0.458846),
                             MoveLR(0.929411),MoveUD(0.223548),Delay(20),MoveLR(0.95294),MoveUD(0.137272),Delay(20),MoveLR(1),MoveUD(-0.0509659),Delay(20),MoveUD(-0.247047),Delay(20),MoveUD(-0.443129),Delay(20),MoveUD(-0.694113),MoveUD(-0.921567),
                             Delay(30),MoveLR(0.858821),MoveUD(-1),Delay(10),MoveLR(0.68627),Delay(10),MoveLR(0.364696),Delay(20),MoveLR(0.0509659),Delay(20),MoveLR(-0.223548),Delay(10),MoveLR(-0.600024),MoveUD(-0.913724),Delay(30),MoveLR(-0.858852),MoveUD(-0.717643),
                             MoveLR(-1),MoveUD(-0.474502),Delay(20),MoveUD(-0.396069),Delay(20),MoveUD(-0.286264),Delay(20),MoveUD(-0.137242),Delay(20),MoveUD(0.0353099),Delay(10),MoveUD(0.32551),Delay(20),MoveUD(0.592181),Delay(10),MoveUD(0.851009),Delay(10),MoveUD(1),
                             Delay(30),MoveLR(-0.764733),Delay(20),MoveLR(-0.403943),MoveLR(-0.145116),Delay(30),MoveLR(0.0901822),MoveLR(0.32548),Delay(30),MoveLR(0.560778),MoveUD(0.929441),Delay(20),MoveLR(0.709799),MoveUD(0.73336),MoveLR(0.803919),
                             MoveUD(0.545122),Delay(20),MoveLR(0.882351),MoveUD(0.356883),Delay(10),MoveLR(0.968627),MoveUD(0.113742),Delay(20),MoveLR(0.992157),MoveUD(-0.0823389),Delay(30),MoveUD(-0.309793),Delay(10),MoveUD(-0.545091),Delay(20),MoveLR(0.882351),
                             MoveUD(-0.874508),Delay(20),MoveLR(0.756859),MoveUD(-1),Delay(10),MoveLR(0.576464),Delay(20),MoveLR(0.254891),Delay(10),MoveLR(-0.0274667),Delay(10),MoveLR(-0.356883),Delay(30),MoveLR(-0.592181),MoveLR(-0.827479),MoveUD(-0.921567),
                             Delay(20),MoveLR(-1),MoveUD(-0.749016),Delay(20),MoveUD(-0.61568),Delay(10),MoveUD(-0.403912),Delay(20),MoveUD(-0.207831),Delay(10),MoveUD(0.121586),Delay(30),MoveUD(0.34904),Delay(10),MoveUD(0.560808),Delay(10),MoveUD(0.827479),Delay(30),MoveUD(1),
                             Delay(20),MoveLR(-0.976501),MoveLR(-0.670614),Delay(20),MoveLR(-0.239235),Delay(20),MoveLR(0.160772),Delay(20),MoveLR(0.443129),Delay(10),MoveLR(0.68627),MoveUD(0.976501),Delay(30),MoveLR(0.929411),MoveUD(0.73336),MoveLR(1),
                             MoveUD(0.482376),Delay(20),MoveUD(0.34904),Delay(10),MoveUD(0.160802),Delay(30),MoveUD(-0.0744957),Delay(10),MoveUD(-0.333323),Delay(20),MoveUD(-0.647053),Delay(20),MoveUD(-0.937254),Delay(10),MoveLR(0.858821),MoveUD(-1),Delay(10),MoveLR(0.576464),
                             Delay(30),MoveLR(0.184301),Delay(10),MoveLR(-0.121586),Delay(10),MoveLR(-0.474532),Delay(30),MoveLR(-0.670614),MoveLR(-0.851009),Delay(30),MoveLR(-1),MoveUD(-0.968627),Delay(20),MoveUD(-0.843135),Delay(10),MoveUD(-0.631367),Delay(20),MoveUD(-0.403912),
                             MoveUD(-0.176458),Delay(20),MoveUD(0.0902127),Delay(20),MoveUD(0.380413),Delay(10),MoveUD(0.717673),Delay(30),MoveUD(1),Delay(10),MoveLR(-0.741203),Delay(20),MoveLR(-0.458846),Delay(10),MoveLR(-0.145116),Delay(10),MoveLR(0.0980255),Delay(20),MoveLR(0.294107),
                             Delay(30),MoveLR(0.466659),MoveLR(0.717643),MoveUD(0.796106),Delay(20),MoveLR(0.921567),MoveUD(0.443159),Delay(20),MoveLR(1),MoveUD(0.145116),Delay(10),MoveUD(-0.0274361),Delay(30),MoveUD(-0.223518),MoveUD(-0.427442),
                             Delay(20),MoveUD(-0.874508),Delay(20),MoveUD(-1),Delay(10),MoveLR(0.929411),Delay(20),MoveLR(0.68627),Delay(20),MoveLR(0.364696),Delay(20),MoveLR(0.0431227),Delay(10),MoveLR(-0.333354),Delay(20),MoveLR(-0.639241),Delay(20),MoveLR(-0.968657),MoveUD(-0.968627),
                             Delay(20),MoveLR(-1),MoveUD(-0.890194),MoveUD(-0.866665),Delay(20),MoveUD(-0.749016),Delay(20),MoveUD(-0.529405),Delay(20),MoveUD(-0.30195),Delay(10),MoveUD(-0.00390637),Delay(10),MoveUD(0.262764),Delay(30),MoveLR(-0.600024),MoveUD(0.458846),
                             Delay(10),MoveLR(-0.294137),MoveUD(0.482376),Delay(20),MoveLR(-0.200018),MoveUD(0.505905),Delay(10),MoveLR(-0.145116),MoveUD(0.545122),Delay(20),MoveLR(-0.0353099),MoveUD(0.584338),Delay(20),MoveLR(0.137242),MoveUD(0.592181),
                             Delay(20),MoveLR(0.30195),Delay(10),MoveLR(0.490188),Delay(10),MoveLR(0.599994),MoveUD(0.529435),Delay(30),MoveLR(0.66274),MoveUD(0.3961),Delay(20),MoveLR(0.670583),MoveUD(0.231391),MoveLR(0.68627),MoveUD(0.0745262),

                             Move(0,-0.01),
                             Delay(100),
                             Move(0,0),
                             Delay(1000),

                             Text(R.phrase05Text), # for example when you punch..
                             Delay(500),

                             Move(0,-0.01),
                             Delay(100),
                             Move(0,0),
                             Delay(500),

                             SpawnSpaz(0,(-0.09249162673950195, 4.337906360626221, -2.9293924808502197),makeCurrent=True,flash=False),
                             SpawnSpaz(1,(-3.1, 4.3, -2.9),makeCurrent=False,color=(1,1,0.4),name=R.randomName1Text),
                             Move(-1.0,0),
                             Delay(1000),
                             Move(0,-0.01),
                             Delay(100),
                             Move(0,0),
                             Delay(1000),
                             Text(R.phrase06Text), # your damage is based
                             Delay(1000),
                             Move(-0.05,0),
                             Delay(200),
                             Punch(),
                             Delay(800),
                             Punch(),
                             Delay(800),
                             Punch(),
                             Delay(800),
                             Move(0,-0.01),
                             Delay(100),
                             Move(0,0),
                             Text(R.phrase07Text.replace('${NAME}',R.randomName1Text)), # see that didnt hurt fred
                             Delay(2000),
                             Celebrate('right',spazNum=1),
                             Delay(1400),
                             Text(R.phrase08Text), # lets jump and spin to get more speed

                             Delay(30),MoveLR(0),Delay(40),MoveLR(0),Delay(40),MoveLR(0),Delay(130),MoveLR(0),Delay(100),MoveLR(0),
                             Delay(10),MoveLR(0.0480667),Delay(40),MoveLR(0.056093),MoveLR(0.0681173),Delay(30),MoveLR(0.0801416),Delay(10),MoveLR(0.184301),
                             Delay(10),MoveLR(0.207831),Delay(20),MoveLR(0.231361),Delay(30),MoveLR(0.239204),Delay(30),MoveLR(0.254891),Delay(40),MoveLR(0.270577),
                             Delay(10),MoveLR(0.30195),Delay(20),MoveLR(0.341166),Delay(30),MoveLR(0.388226),MoveLR(0.435286),Delay(30),MoveLR(0.490188),
                             Delay(10),MoveLR(0.560778),Delay(20),MoveLR(0.599994),Delay(10),MoveLR(0.647053),Delay(10),MoveLR(0.68627),Delay(30),MoveLR(0.733329),
                             Delay(20),MoveLR(0.764702),Delay(10),MoveLR(0.827448),Delay(20),MoveLR(0.874508),Delay(20),MoveLR(0.929411),Delay(10),MoveLR(1),
                             Delay(830),MoveUD(0.0274667),Delay(10),MoveLR(0.95294),MoveUD(0.113742),Delay(30),MoveLR(0.780389),MoveUD(0.184332),
                             Delay(10),MoveLR(0.27842),MoveUD(0.0745262),Delay(20),MoveLR(0),MoveUD(0),Delay(390),MoveLR(0),
                             MoveLR(0),Delay(20),MoveLR(0),Delay(20),MoveLR(0),Delay(10),MoveLR(-0.0537431),Delay(20),MoveLR(-0.215705),
                             Delay(30),MoveLR(-0.388256),MoveLR(-0.529435),Delay(30),MoveLR(-0.694143),Delay(20),MoveLR(-0.851009),MoveUD(0.0588397),
                             Delay(10),MoveLR(-1),MoveUD(0.0745262),Run(release=False),Delay(200),MoveUD(0.0509964),Delay(30),MoveUD(0.0117801),
                             Delay(20),MoveUD(-0.0901822),MoveUD(-0.372539),Delay(30),MoveLR(-0.898068),MoveUD(-0.890194),Jump(release=False),
                             Delay(20),MoveLR(-0.647084),MoveUD(-1),MoveLR(-0.427473),Delay(20),MoveLR(-0.00393689),Delay(10),MoveLR(0.537248),
                             Delay(30),MoveLR(1),Delay(50),RunRelease(),JumpRelease(),Delay(50),MoveUD(-0.921567),MoveUD(-0.749016),
                             Delay(30),MoveUD(-0.552934),Delay(10),MoveUD(-0.247047),Delay(20),MoveUD(0.200018),Delay(20),MoveUD(0.670614),MoveUD(1),
                             Delay(70),MoveLR(0.97647),Delay(20),MoveLR(0.764702),Delay(20),MoveLR(0.364696),Delay(20),MoveLR(0.00390637),MoveLR(-0.309824),
                             Delay(20),MoveLR(-0.576495),Delay(30),MoveLR(-0.898068),Delay(10),MoveLR(-1),MoveUD(0.905911),Delay(20),MoveUD(0.498062),
                             Delay(20),MoveUD(0.0274667),MoveUD(-0.403912),Delay(20),MoveUD(-1),Run(release=False),Jump(release=False),
                             Delay(30),Punch(release=False),Delay(70),JumpRelease(),Delay(110),MoveLR(-0.976501),RunRelease(),PunchRelease(),
                             Delay(10),MoveLR(-0.952971),Delay(20),MoveLR(-0.905911),MoveLR(-0.827479),Delay(20),MoveLR(-0.75689),Delay(30),MoveLR(-0.73336),
                             MoveLR(-0.694143),Delay(20),MoveLR(-0.670614),Delay(30),MoveLR(-0.66277),Delay(10),MoveUD(-0.960784),Delay(20),MoveLR(-0.623554),
                             MoveUD(-0.874508),Delay(10),MoveLR(-0.545122),MoveUD(-0.694113),Delay(20),MoveLR(-0.505905),MoveUD(-0.474502),
                             Delay(20),MoveLR(-0.458846),MoveUD(-0.356853),MoveLR(-0.364727),MoveUD(-0.27842),Delay(20),MoveLR(0.00390637),

                             Move(0,0),
                             Delay(1000),
                             Text(R.phrase09Text), # ah thats better
                             Delay(1900),
                             Text(R.phrase10Text),
                             Delay(100),
                             SpawnSpaz(0,(-3.170698404312134, 4.337471961975098, -4.080751419067383),makeCurrent=True,flash=False),
                             SpawnSpaz(1,(2.4273335933685303, 4.2, -4.9),makeCurrent=False,color=(0.9,0.5,1.0),name=R.randomName2Text),
                             Delay(1800),
                             Text(R.phrase11Text), # hold ANY button to run
                             Delay(300),

                             MoveUD(0),Delay(20),MoveUD(-0.0520646),Delay(20),MoveLR(0),MoveUD(-0.223518),Run(release=False),Jump(release=False),Delay(10),MoveLR(0.0980255),MoveUD(-0.309793),Delay(30),MoveLR(0.160772),MoveUD(-0.427442),
                             Delay(20),MoveLR(0.231361),MoveUD(-0.545091),Delay(10),MoveLR(0.317637),MoveUD(-0.678426),Delay(20),MoveLR(0.396069),MoveUD(-0.819605),MoveLR(0.482345),MoveUD(-0.913724),Delay(20),MoveLR(0.560778),MoveUD(-1),
                             Delay(20),MoveLR(0.607837),Delay(10),MoveLR(0.623524),Delay(30),MoveLR(0.647053),Delay(20),MoveLR(0.670583),MoveLR(0.694113),Delay(30),MoveLR(0.733329),Delay(20),MoveLR(0.764702),MoveLR(0.788232),Delay(20),MoveLR(0.827448),Delay(10),MoveLR(0.858821),
                             Delay(20),MoveLR(0.921567),Delay(30),MoveLR(0.97647),MoveLR(1),Delay(130),MoveUD(-0.960784),Delay(20),MoveUD(-0.921567),Delay(30),MoveUD(-0.866665),MoveUD(-0.819605),Delay(30),MoveUD(-0.772546),MoveUD(-0.725486),Delay(30),MoveUD(-0.631367),
                             Delay(10),MoveUD(-0.552934),Delay(20),MoveUD(-0.474502),Delay(10),MoveUD(-0.403912),Delay(30),MoveUD(-0.356853),Delay(30),MoveUD(-0.34901),Delay(20),MoveUD(-0.333323),Delay(20),MoveUD(-0.32548),Delay(10),MoveUD(-0.30195),Delay(20),MoveUD(-0.27842),Delay(30),MoveUD(-0.254891),
                             MoveUD(-0.231361),Delay(30),MoveUD(-0.207831),Delay(20),MoveUD(-0.199988),MoveUD(-0.176458),Delay(30),MoveUD(-0.137242),MoveUD(-0.0823389),Delay(20),MoveUD(-0.0274361),Delay(20),MoveUD(0.00393689),Delay(40),MoveUD(0.0353099),Delay(20),MoveUD(0.113742),
                             Delay(10),MoveUD(0.137272),Delay(20),MoveUD(0.160802),MoveUD(0.184332),Delay(20),MoveUD(0.207862),Delay(30),MoveUD(0.247078),MoveUD(0.262764),Delay(20),MoveUD(0.270608),Delay(30),MoveUD(0.294137),MoveUD(0.32551),Delay(30),MoveUD(0.37257),
                             Celebrate('left',1),
                             Delay(20),MoveUD(0.498062),MoveUD(0.560808),Delay(30),MoveUD(0.654927),MoveUD(0.694143),Delay(30),MoveUD(0.741203),Delay(20),MoveUD(0.780419),MoveUD(0.819636),Delay(20),MoveUD(0.843165),Delay(20),MoveUD(0.882382),Delay(10),MoveUD(0.913755),
                             Delay(30),MoveUD(0.968657),MoveUD(1),Delay(540),Punch(release=False),Delay(210),MoveUD(0.968657),Delay(30),MoveUD(0.75689),PunchRelease(),Delay(20),MoveLR(0.95294),MoveUD(0.435316),RunRelease(),JumpRelease(),
                             MoveLR(0.811762),MoveUD(0.270608),Delay(20),MoveLR(0.670583),MoveUD(0.160802),Delay(20),MoveLR(0.466659),MoveUD(0.0588397),Delay(10),MoveLR(0.317637),MoveUD(-0.00390637),Delay(20),MoveLR(0.0801416),Delay(10),MoveLR(0),
                             Delay(20),MoveLR(0),Delay(30),MoveLR(0),Delay(30),MoveLR(0),Delay(20),MoveLR(0),Delay(100),MoveLR(0),Delay(30),MoveUD(0),Delay(30),MoveUD(0),Delay(50),MoveUD(0),MoveUD(0),Delay(30),MoveLR(0),
                             MoveUD(-0.0520646),MoveLR(0),MoveUD(-0.0640889),Delay(20),MoveLR(0),MoveUD(-0.0881375),Delay(30),MoveLR(-0.0498978),MoveUD(-0.199988),MoveLR(-0.121586),MoveUD(-0.207831),Delay(20),MoveLR(-0.145116),
                             MoveUD(-0.223518),Delay(30),MoveLR(-0.152959),MoveUD(-0.231361),MoveLR(-0.192175),MoveUD(-0.262734),Delay(30),MoveLR(-0.200018),MoveUD(-0.27842),Delay(20),MoveLR(-0.239235),MoveUD(-0.30195),MoveUD(-0.309793),
                             Delay(40),MoveUD(-0.333323),Delay(10),MoveUD(-0.34901),Delay(30),MoveUD(-0.372539),MoveUD(-0.396069),Delay(20),MoveUD(-0.443129),Delay(20),MoveUD(-0.458815),Delay(10),MoveUD(-0.474502),Delay(50),MoveUD(-0.482345),Delay(30),MoveLR(-0.215705),Delay(30),MoveLR(-0.200018),
                             Delay(10),MoveLR(-0.192175),Delay(10),MoveLR(-0.176489),Delay(30),MoveLR(-0.152959),Delay(20),MoveLR(-0.145116),MoveLR(-0.121586),MoveUD(-0.458815),Delay(30),MoveLR(-0.098056),MoveUD(-0.419599),Delay(10),MoveLR(-0.0745262),MoveUD(-0.333323),
                             Delay(10),MoveLR(0.00390637),MoveUD(0),Delay(990),MoveLR(0),Delay(660),MoveUD(0),

                             Text(R.phrase12Text), # for extra-awesome punches,...
                             Delay(200),
                             SpawnSpaz(0,(2.368781805038452, 4.337533950805664, -4.360159873962402),makeCurrent=True,flash=False),
                             SpawnSpaz(1,(-3.2, 4.3, -4.5),makeCurrent=False,color=(1.0,0.7,0.3),name=R.randomName3Text),

                             Delay(100),
                             Powerup(1,(2.5,0.0,0),relativeTo=0),
                             Move(1,0),
                             Delay(1700),
                             Move(0,-0.1),
                             Delay(100),
                             Move(0,0),
                             Delay(500),

                             Delay(320),MoveLR(0),Delay(20),MoveLR(0),Delay(10),MoveLR(0),Delay(20),MoveLR(-0.333354),MoveLR(-0.592181),Delay(20),MoveLR(-0.788263),Delay(20),MoveLR(-1),MoveUD(0.0353099),MoveUD(0.0588397),Delay(10),Run(release=False),
                             Delay(510),MoveUD(0.0274667),MoveUD(0.00393689),Delay(10),MoveUD(-0.00390637),Delay(440),MoveUD(0.0353099),Delay(20),MoveUD(0.0588397),Delay(10),MoveUD(0.0902127),Delay(260),MoveUD(0.0353099),Delay(30),MoveUD(0.00393689),Delay(10),MoveUD(-0.00390637),MoveUD(-0.0274361),
                             Celebrate('left',1),
                             Delay(10),MoveUD(-0.0823389),Delay(30),MoveUD(-0.176458),MoveUD(-0.286264),Delay(20),MoveUD(-0.498032),Jump(release=False),MoveUD(-0.764702),Delay(30),MoveLR(-0.858852),MoveUD(-1),MoveLR(-0.780419),Delay(20),MoveLR(-0.717673),
                             Delay(10),MoveLR(-0.552965),Delay(10),MoveLR(-0.341197),Delay(10),MoveLR(-0.0274667),Delay(10),MoveLR(0.27842),Delay(20),MoveLR(0.811762),MoveLR(1),RunRelease(),JumpRelease(),Delay(260),MoveLR(0.95294),Delay(30),MoveLR(0.756859),
                             Delay(10),MoveLR(0.317637),MoveLR(-0.00393689),Delay(10),MoveLR(-0.341197),Delay(30),MoveLR(-0.647084),MoveUD(-0.921567),Delay(10),MoveLR(-1),MoveUD(-0.599994),MoveUD(-0.474502),Delay(10),MoveUD(-0.309793),Delay(30),MoveUD(-0.160772),
                             MoveUD(-0.0352794),Delay(10),MoveUD(0.176489),Delay(20),MoveUD(0.607868),Run(release=False),Jump(release=False),Delay(20),MoveUD(1),Delay(30),MoveLR(-0.921598),Delay(10),MoveLR(-0.639241),Delay(20),MoveLR(-0.223548),Punch(release=False),
                             Delay(10),MoveLR(0.254891),Delay(10),MoveLR(0.741172),MoveLR(1),Delay(40),JumpRelease(),Delay(40),MoveUD(0.976501),Delay(10),MoveUD(0.73336),Delay(10),MoveUD(0.309824),Delay(20),MoveUD(-0.184301),Delay(20),MoveUD(-0.811762),MoveUD(-1),
                             KillSpaz(1,explode=True),
                             Delay(10),RunRelease(),PunchRelease(),Delay(110),MoveLR(0.97647),MoveLR(0.898038),Delay(20),MoveLR(0.788232),Delay(20),MoveLR(0.670583),Delay(10),MoveLR(0.505875),Delay(10),MoveLR(0.32548),Delay(20),MoveLR(0.137242),Delay(10),MoveLR(-0.00393689),
                             Delay(10),MoveLR(-0.215705),MoveLR(-0.356883),Delay(20),MoveLR(-0.451003),Delay(10),MoveLR(-0.552965),Delay(20),MoveLR(-0.670614),MoveLR(-0.780419),Delay(10),MoveLR(-0.898068),Delay(20),MoveLR(-1),Delay(370),MoveLR(-0.976501),Delay(10),MoveLR(-0.952971),
                             Delay(10),MoveLR(-0.929441),MoveLR(-0.898068),Delay(30),MoveLR(-0.874538),Delay(10),MoveLR(-0.851009),Delay(10),MoveLR(-0.835322),MoveUD(-0.968627),Delay(10),MoveLR(-0.827479),MoveUD(-0.960784),Delay(20),MoveUD(-0.945097),Delay(70),MoveUD(-0.937254),
                             Delay(20),MoveUD(-0.913724),Delay(20),MoveUD(-0.890194),MoveLR(-0.780419),MoveUD(-0.827448),Delay(20),MoveLR(0.317637),MoveUD(0.3961),MoveLR(0.0195929),MoveUD(0.056093),Delay(20),MoveUD(0),Delay(750),MoveLR(0),

                             Text(R.phrase13Text.replace('${NAME}',R.randomName3Text)), # whoops sorry bill
                             RemoveGloves(),
                             Delay(2000),

                             Text(R.phrase14Text.replace('${NAME}',R.randomName4Text)), # you can pick up and throw things such as chuck here
                             SpawnSpaz(1,(5,0,-1.0),relativeTo=0,makeCurrent=False,color=(0.4,1.0,0.7),name=R.randomName4Text),
                             Delay(1000),
                             Celebrate('left',1,duration=1000),
                             Move(1,0.2),
                             Delay(1900),
                             PickUp(),
                             Delay(200),
                             Move(0.5,1.0),
                             Delay(1200),
                             PickUp(),
                             Move(0,0),
                             Delay(1000),
                             Celebrate('left'),
                             Delay(1500),
                             Move(0,-1.0),
                             Delay(800),
                             Move(0,0),
                             Delay(800),
                             Text(R.phrase15Text), # lastly theres bombs
                             Delay(1900),
                             Text(R.phrase16Text), # throwing bombs takes practice
                             Delay(2000),
                             Bomb(),
                             Move(-0.1,-0.1),
                             Delay(100),
                             Move(0,0),
                             Delay(500),
                             Delay(1000),
                             Bomb(),
                             Delay(2000),
                             Text(R.phrase17Text), # not a very good throw
                             Delay(3000),
                             Text(R.phrase18Text), # moving helps you get distance
                             Delay(1000),
                             Bomb(),
                             Delay(500),
                             Move(-0.3,0),
                             Delay(100),
                             Move(-0.6,0),
                             Delay(100),
                             Move(-1,0),
                             Delay(800),
                             Bomb(),
                             Delay(400),
                             Move(0,-0.1),
                             Delay(100),
                             Move(0,0),
                             Delay(2000),
                             Text(R.phrase19Text), # jumping helps you get height
                             Delay(2000),
                             Bomb(),
                             Delay(500),
                             Move(1,0),
                             Delay(300),
                             Jump(releaseDelay=250),
                             Delay(500),
                             Jump(releaseDelay=250),
                             Delay(550),
                             Jump(releaseDelay=250),
                             Delay(160),
                             Punch(),
                             Delay(500),
                             Move(0,-0.1),
                             Delay(100),
                             Move(0,0),
                             Delay(1000),
                             Text(R.phrase20Text), # whiplash your bombs
                             Delay(1000),

            Bomb(release=False),Delay(80),RunRelease(),BombRelease(),Delay(620),MoveLR(0),Delay(10),MoveLR(0),Delay(40),MoveLR(0),Delay(10),MoveLR(-0.0537431),MoveUD(0),Delay(20),MoveLR(-0.262764),Delay(20),MoveLR(-0.498062),
            Delay(10),MoveLR(-0.639241),Delay(20),MoveLR(-0.73336),Delay(10),MoveLR(-0.843165),MoveUD(-0.0352794),Delay(30),MoveLR(-1),Delay(10),MoveUD(-0.0588092),Delay(10),MoveUD(-0.160772),Delay(20),MoveUD(-0.286264),Delay(20),MoveUD(-0.427442),Delay(10),MoveUD(-0.623524),
            Delay(20),MoveUD(-0.843135),Delay(10),MoveUD(-1),Delay(40),MoveLR(-0.890225),Delay(10),MoveLR(-0.670614),Delay(20),MoveLR(-0.435316),Delay(20),MoveLR(-0.184332),Delay(10),MoveLR(0.00390637),Delay(20),MoveLR(0.223518),Delay(10),MoveLR(0.388226),Delay(20),MoveLR(0.560778),
            Delay(20),MoveLR(0.717643),Delay(10),MoveLR(0.890194),Delay(20),MoveLR(1),Delay(30),MoveUD(-0.968627),Delay(20),MoveUD(-0.898038),Delay(10),MoveUD(-0.741172),Delay(20),MoveUD(-0.498032),Delay(20),MoveUD(-0.247047),Delay(10),MoveUD(0.00393689),Delay(20),MoveUD(0.239235),
            Delay(20),MoveUD(0.458846),Delay(10),MoveUD(0.70983),Delay(30),MoveUD(1),Delay(10),MoveLR(0.827448),Delay(10),MoveLR(0.678426),Delay(20),MoveLR(0.396069),Delay(10),MoveLR(0.0980255),Delay(20),MoveLR(-0.160802),Delay(20),MoveLR(-0.388256),Delay(10),MoveLR(-0.545122),
            Delay(30),MoveLR(-0.73336),Delay(10),MoveLR(-0.945128),Delay(10),MoveLR(-1),Delay(50),MoveUD(0.960814),Delay(20),MoveUD(0.890225),Delay(10),MoveUD(0.749046),Delay(20),MoveUD(0.623554),Delay(20),MoveUD(0.498062),Delay(10),MoveUD(0.34904),Delay(20),MoveUD(0.239235),
            Delay(20),MoveUD(0.137272),Delay(10),MoveUD(0.0117801),Delay(20),MoveUD(-0.0117496),Delay(10),MoveUD(-0.0274361),Delay(90),MoveUD(-0.0352794),Run(release=False),Jump(release=False),Delay(80),Punch(release=False),Delay(60),MoveLR(-0.968657),Delay(20),MoveLR(-0.835322),
            Delay(10),MoveLR(-0.70983),JumpRelease(),Delay(30),MoveLR(-0.592181),MoveUD(-0.0588092),Delay(10),MoveLR(-0.490219),MoveUD(-0.0744957),Delay(10),MoveLR(-0.41963),Delay(20),MoveLR(0),MoveUD(0),Delay(20),MoveUD(0),
                             PunchRelease(),
                             RunRelease(),
                             Delay(500),
                             Move(0,-0.1),
                             Delay(100),
                             Move(0,0),
                             Delay(2000),
                             Text(R.phrase21Text), # timing your bombs can be tricky
                             Move(-1,0),
                             Delay(1000),
                             Move(0,-0.1),
                             Delay(100),
                             Move(0,0),
                             SpawnSpaz(1,(6.5,0,-0.75),relativeTo=0,makeCurrent=False,color=(0.3,0.8,1.0),name=R.randomName5Text),
                             Delay(1000),
                             Move(-1,0),
                             Delay(1200),
                             Bomb(),
                             Move(0,0),
                             Delay(300),
                             Move(1,0),
                             Delay(300),
                             Jump(),
                             Delay(100),
                             Punch(),
                             Delay(800),
                             Move(-1,0),
                             Delay(1000),
                             Move(0,0),
                             Delay(1000),
                             Text(R.phrase22Text), # dang
                             Delay(2000),
                             Text(''),
                             Delay(200),
                             Text(R.phrase23Text), # try cooking off
                             Delay(1500),
                             Bomb(),
                             Delay(800),
                             Move(1,0.1),
                             Delay(1100),
                             Jump(),
                             Delay(100),
                             Punch(),
                             Delay(100),
                             Move(0,-0.1),
                             Delay(100),
                             Move(0,0),
                             Delay(2000),
                             Text(R.phrase24Text), # hooray nicely cooked
                             Celebrate(),
                             Delay(2000),
                             Text(""),
                             Move(0.5,-0.5),
                             Delay(1000),
                             Move(0,-0.1),
                             Delay(100),
                             Move(0,0),
                             Delay(1000),
                             Text(R.phrase25Text), # well thats just about it
                             Delay(2000),
                             Text(R.phrase26Text), # go get em tiger
                             Delay(2000),
                             Text(R.phrase27Text), # remember you training
                             Delay(3000),
                             Text(R.phrase28Text), # well maybe
                             Delay(1600),
                             Text(R.phrase29Text), # good luck
                             Celebrate('right',duration=10000),
                             Delay(1000),
                             End(),
                             Delay(10000),
                             Reset(),
                         ]

        except Exception,e:
            import traceback
            traceback.print_exc(e)

        # if we read some, exec them..
        if self._entries:
            self._runNextEntry()
        # otherwise try again in a few seconds..
        else:
            #print 'no entries read; will try again soon...'
            self._readEntriesTimer = bs.Timer(3000,bs.WeakCall(self._readEntries))

        
    def _runNextEntry(self):
        
        while self._entries:

            # if our test file mod time has changed..
            # modTime = os.path.getmtime(self._testFile)
            # if modTime != self._testFileModTime:
            #     self._testFileModTime = modTime
            #     break
            
            entry = self._entries.pop(0)
            try:
                result = entry.run(self)
            except Exception,e:
                import traceback
                traceback.print_exc(e)
                break
                
            # if the entry returns an int value, set a timer..
            # otherwise just keep going..
            if result is not None:
                self._entryTimer = bs.Timer(result,bs.WeakCall(self._runNextEntry))
                return
            
        # done with these entries.. start over soon
        self._readEntriesTimer = bs.Timer(1000,bs.WeakCall(self._readEntries))
        
    def _updateSkipVotes(self):
        count = sum(1 for player in self.players if player.gameData['pressed'])
        self._skipCountText.text = self._R.skipVoteCountText.replace('${COUNT}',str(count)).replace('${TOTAL}',str(len(self.players))) if count > 0 else ''
        if count >= len(self.players) and len(self.players) > 0 and not self._haveSkipped:
            self._haveSkipped = True
            bs.playSound(bs.getSound('swish'))
            self._skipCountText.text = self._R.skippingText
            self._skipText.text = ''
            self.end()
            

    def _playerPressedButton(self,player):
        player.gameData['pressed'] = True

        # test...
        if not all(player.exists() for player in self.players):
            bs.printErrorOnce("Nonexistant player in _playerPressedButton: "+str([str(p) for p in self.players])+': we are '+str(player))

        self._updateSkipVotes()
        
    def onPlayerJoin(self,player):
        bs.Activity.onPlayerJoin(self,player)
        player.gameData['pressed'] = False
        # we just wanna know if this player presses anything..
        player.assignInput(('jumpPress','punchPress','bombPress','pickUpPress'),bs.WeakCall(self._playerPressedButton,player))

    def onPlayerLeave(self,player):

        # test...
        if not all(player.exists() for player in self.players):
            bs.printErrorOnce("Nonexistant player in onPlayerLeave: "+str([str(p) for p in self.players])+': we are '+str(player))

        bs.Activity.onPlayerLeave(self,player)
        # our leaving may influence the vote total needed/etc
        self._updateSkipVotes()
        
        # if all our players leave and we're in a co-op session lets leave the session
        # (otherwise we might lead into a co-op game with zero players or something)
        # if len(self.players) == 0 and isinstance(self.getSession,bs.CoopSession):
        #     self.getSession().end()

    def __del__(self):
        bs.Activity.__del__(self)
